/*
 *  ssp.h
 *
 *  Copyright (C) 2004 Ray Lehtiniemi, All Rights Reserved.
 *  Copyright (C) 2004 Siconix, Inc., All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef SSP_H
#define SSP_H

#include <linux/list.h>
#include <linux/completion.h>
#include <asm/semaphore.h>

struct ssp_master_ops;
struct ssp_slave;

struct ssp_transfer {

	/* how many _bytes_ per unit of data */
	int size;

	/* how many units of data left to be clocked out */
	int len;

	/* how many units of data left to be clocked in */
	int rlen;

	/* buffer of units to shift out to slave */
	void * tx;

	/* buffer to hold units shifted in from slave */
	void * rx;

	/* signaled when complete */
	struct completion * c;
};

struct ssp_master {

	/* serialize access to the frontend */
	struct semaphore sem;

	/* synchronize the frontend and the backend */
	struct completion completion;

	/* the slave devices on this master */
	struct list_head slave;

	/* the active transfer on the bus  */
	struct ssp_transfer transfer;

	/* methods for different master controllers */
	struct ssp_master_ops * ops;
};

struct ssp_master_ops {

	/* configure the master for accessing a particular slave */
	int (*configure) (struct ssp_slave *);

	/* enable the backend to start the transfer */
	int (*enable) (struct ssp_transfer *);

	/* disable the backend to stop the transfer */
	int (*disable) (void);
};

struct ssp_slave {

	/* our link in the masters list of slaves */
	struct list_head list;

	/* our master */
	struct ssp_master * master;
#if 0
	/* the upper level protocol spoken by the device */
	struct ssp_protocol * protocol;
#endif
	/* the phase and polarity modes we support */
	unsigned long supported_modes;

	/* our maximum clock rate, in Hz */
	unsigned long max_clock_rate;

	/* our data unit size, in bits */
	int word_size;

	/* our data unit size, in bytes */
	int nbytes;

	/* assert or negate the chip select line for this device */
	void (*chip_select) (struct ssp_slave *, int);
};


/* manage the master-slave list */
extern int  ssp_add_slave (struct ssp_master *, struct ssp_slave *);
extern void ssp_remove_slave (struct ssp_master *, struct ssp_slave *);

/* perform a single transaction */
extern int  ssp_do (struct ssp_slave *s, int len, void *tx, void *rx);

/* perform each step of a transaction */
extern int  ssp_start (struct ssp_slave *s);      // grab master, blocking
extern int  ssp_trystart (struct ssp_slave *s);   // grab master, non-blocking
extern void ssp_select (struct ssp_slave *s);     // assert CS
extern int  ssp_post (struct ssp_slave *s, int len, void *tx, void *rx); // start a transfer
extern int  ssp_wait (struct ssp_slave *s);       // wait for completion
//extern int  ssp_disable (struct ssp_slave *s);    // stop a transfer
extern void ssp_deselect (struct ssp_slave *s);   // negate CS
extern void ssp_finish (struct ssp_slave *s);     // release master

/* module stuff */
extern int ssp_init (void);
extern void ssp_exit (void);

#endif
